<?php

declare(strict_types=1);

namespace Drupal\rdf_sync;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use EasyRdf\Http\Exception as EasyRdfException;
use EasyRdf\Sparql\Client;
use EasyRdf\Sparql\Result;

/**
 * Defines rdf_sync.connection service.
 */
class RdfSyncConnection implements RdfSyncConnectionInterface {

  /**
   * EasyRdf client.
   */
  protected Client $client;

  /**
   * The rdf_sync settings.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected Config $config;

  /**
   * Constructs Rdf Sync Connection object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Config factory.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->config = $configFactory->get('rdf_sync.settings');
  }

  /**
   * {@inheritdoc}
   */
  public function query(string $query): Result {
    return $this->performOperation($query, 'query');
  }

  /**
   * Provides a helper to run a query or an update.
   *
   * @param string $query
   *   The SPARQL query.
   * @param string $method
   *   The method: 'query' or 'update'.
   *
   * @return \EasyRdf\Sparql\Result
   *   The result object.
   */
  protected function performOperation(string $query, string $method): Result {
    assert(in_array($method, ['query', 'update'], TRUE));

    try {
      return $this->getClient()->$method($query);
    }
    catch (EasyRdfException $exception) {
      // Re-throw the exception, but with the query as message.
      throw new \Exception("Execution of $method failed with message '{$exception->getBody()}'. Query: " . $query, $exception->getCode(), $exception);
    }
  }

  /**
   * Returns the Easy RDF SPARQL client.
   *
   * @return \EasyRdf\Sparql\Client
   *   The Easy RDF SPARQL client.
   */
  protected function getClient(): Client {
    if (!isset($this->client)) {
      $this->client = new Client($this->getEndpointUrl('query'), $this->getEndpointUrl('update'));
    }
    return $this->client;
  }

  /**
   * {@inheritdoc}
   */
  public function getEndpointUrl(string $type): string {
    assert(in_array($type, ['query', 'update', 'graph_store'], TRUE));

    $endpoint = $this->config->get('endpoint');
    $path = trim($endpoint['paths'][$type], '/');

    return "{$endpoint['scheme']}://{$endpoint['host']}:{$endpoint['port']}/$path";
  }

  /**
   * {@inheritdoc}
   */
  public function update(string $query): Result {
    return $this->performOperation($query, 'update');
  }

}
