<?php

declare(strict_types=1);

namespace Drupal\Tests\rdf_sync\Traits;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\node\Entity\Node;
use Drupal\taxonomy\Entity\Term;
use Drupal\Tests\field\Traits\EntityReferenceFieldCreationTrait;
use PHPUnit\Framework\Assert;

/**
 * Tests reusable code.
 */
trait RdfSyncTestTrait {

  use EntityReferenceFieldCreationTrait;

  /**
   * Sets up the tests.
   */
  protected function setUpFields(): void {
    if (!FieldStorageConfig::loadByName('node', 'body')) {
      // Add a body field.
      FieldStorageConfig::create([
        'entity_type' => 'node',
        'type' => 'text_with_summary',
        'field_name' => 'body',
        'cardinality' => 1,
      ])->save();
    }
    FieldConfig::create([
      'entity_type' => 'node',
      'bundle' => 'page',
      'field_name' => 'body',
      'label' => $this->randomString(),
    ])
      // Using the computed 'processed' column instead of 'value' to get the
      // text format applied.
      ->setThirdPartySetting('rdf_sync', 'processed', [
        'predicate' => 'http://example.com/page/body',
        'type' => NULL,
      ])->save();

    $this->createEntityReferenceField('node', 'page', 'category', $this->randomString(), 'taxonomy_term', 'default', ['target_bundles' => ['category']]);
    FieldConfig::loadByName('node', 'page', 'category')
      ->setThirdPartySetting('rdf_sync', 'target_id', [
        'predicate' => 'http://example.com/page/category',
        'type' => 'resource',
      ])->save();

    // Start on clean.
    \Drupal::service('entity_field.manager')->clearCachedFieldDefinitions();
    \Drupal::service('entity_type.bundle.info')->clearCachedBundles();
  }

  /**
   * Creates testing entities.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface[]
   *   A list of testing entities.
   */
  protected static function createEntities(): array {
    $term = Term::create([
      'vid' => 'category',
      'name' => 'Series',
      'description' => [
        'value' => '<b>7</b> seasons',
        'format' => 'html',
      ],
      'parent' => [
        Term::create([
          'vid' => 'category',
          'name' => 'Series',
          'uri' => 'http://example.com/category/parent/1',
        ]),
        Term::create([
          'vid' => 'category',
          'name' => 'Series',
          'uri' => 'http://example.com/category/parent/2',
        ]),
      ],
      'uri' => 'http://example.com/category/id/1',
    ]);
    $term->save();
    $node = Node::create([
      'type' => 'page',
      'title' => 'Mad Men',
      'uri' => 'http://example.com/page/id/1',
      'body' => [
        'value' => '<strong>Strong</strong> is stripped while <em>Emphasis</em> is kept',
        'format' => 'html',
      ],
      'category' => $term,
    ]);
    $node->save();
    return [$term, $node];
  }

  /**
   * Asserts that an RDF graph contains a list of triples.
   *
   * @param string $graph
   *   The graph URI.
   * @param string[] $triples
   *   A list of triples. Each entry is an 'ntriples' serialized triple.
   */
  protected function assertTriples(string $graph, array $triples): void {
    $this->triplesAssertionHelper(TRUE, $graph, $triples);
  }

  /**
   * Asserts that an RDF graph contains none from a list of triples.
   *
   * @param string $graph
   *   The graph URI.
   * @param string[] $triples
   *   A list of triples. Each entry is an 'ntriples' serialized triple.
   */
  protected function assertNoTriples(string $graph, array $triples): void {
    $this->triplesAssertionHelper(FALSE, $graph, $triples);
  }

  /**
   * Provides a helper fro triples assertion methods.
   *
   * @param bool $checkExistence
   *   TRUE to check the existence, FALSE the lack of triples.
   * @param string $graph
   *   The graph URI.
   * @param string[] $triples
   *   A list of triples. Each entry is an 'ntriples' serialized triple.
   */
  protected function triplesAssertionHelper(bool $checkExistence, string $graph, array $triples): void {
    // Make sure that synchronization has been performed. On the real site this
    // is performed automatically, at the end of the request, but in tests, the
    // request lasts until the end of test, thus we need to trigger it manually.
    \Drupal::service('rdf_sync.synchronizer')->destruct();

    /** @var \Drupal\rdf_sync\RdfSyncConnectionInterface $connection */
    $connection = \Drupal::service('rdf_sync.connection');

    $failures = [];
    foreach ($triples as $triple) {
      $tripeExists = $connection->query("ASK WHERE { GRAPH <$graph> { $triple } }")->getBoolean();
      if ($checkExistence && !$tripeExists) {
        $failures[] = $triple;
      }
    }
    $verb = $checkExistence ? 'missing from' : 'present in';
    Assert::assertEmpty($failures, "Triples $verb the triplestore:\n- " . implode("\n- ", $failures));
  }

  /**
   * {@inheritdoc}
   */
  public function tearDown(): void {
    /** @var \Drupal\rdf_sync\RdfSyncConnectionInterface $connection */
    $connection = \Drupal::service('rdf_sync.connection');

    $connection->query('CLEAR GRAPH <http://data>');
    parent::tearDown();
  }

}
